/*
 * ________________________________________________________________________________________________________
 * Copyright (c) 2020 GebraBit Inc. All rights reserved.
 *
 * This software, related documentation and any modifications thereto (collectively �Software�) is subject
 * to GebraBit and its licensors' intellectual property rights under U.S. and international copyright
 * and other intellectual property rights laws. 
 *
 * GebraBit and its licensors retain all intellectual property and proprietary rights in and to the Software
 * and any use, reproduction, disclosure or distribution of the Software without an express license agreement
 * from GebraBit is strictly prohibited.
 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
 * NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT IN  
 * NO EVENT SHALL GebraBit BE LIABLE FOR ANY DIRECT, SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, 
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THE SOFTWARE.
 * @Author       	: Sepehr Azimi
 * ________________________________________________________________________________________________________
 */
#ifndef _MS5611__H_
#define _MS5611__H_
#include "Arduino.h"
#include "stdint.h"
#include "SPI.h"
#include "math.h"

#define SPI_CS_Pin 10
/************************************************
 *              USER REGISTER MAP               *
 ***********************************************/ 
#define MS5611_RESET                          (0x1E)
#define MS5611_PRESSURE_SAMPLING_START        (0x40)
#define MS5611_TEMPERATURE_SAMPLING_START     (0x50)
#define MS5611_ADC_READ                       (0x00)
#define MS5611_PROM_READ                      (0xA0)
/*----------------------------------------------*
 *           USER REGISTER MAP End              *
 *----------------------------------------------*/
 /************************************************
 *          Register Values Begin                *
 ***********************************************/ 
#define MS5611_OSR_256_CONVERSION_TIME        1
#define MS5611_OSR_512_CONVERSION_TIME        2
#define MS5611_OSR_1024_CONVERSION_TIME       3
#define MS5611_OSR_2048_CONVERSION_TIME       5
#define MS5611_OSR_4096_CONVERSION_TIME       9

#define ADC_DATA_BUFFER_SIZE                  3
#define PROM_DATA_BUFFER_SIZE                 16
#define SEA_LEVEL_PRESSURE                    101325
/*----------------------------------------------*
 *           Register Values End                *
 *----------------------------------------------*/
/**************************************************************
 *       						 Values For Sample Rate    						    *
 **************************************************************/ 
typedef enum {
    OSR_256 = 0x00,
    OSR_512 = 0x02,
    OSR_1024 = 0x04,
    OSR_2048 = 0x06,
    OSR_4096 = 0x08
} MS5611_Output_Sample_Rate;
 /*************************************************
 *  Defining MS5611 Register & Data As Struct   *
 **************************************************/
typedef struct MS5611 {
    uint8_t  PROM_DATA[PROM_DATA_BUFFER_SIZE];
    uint16_t FACTORY_DATA;
    uint16_t C1, C2, C3, C4, C5, C6;
    uint16_t CRC_SERIAL_CODE;
    uint8_t  ADC_DATA[ADC_DATA_BUFFER_SIZE];
    uint32_t ADC_RAW_PRESSURE;
    uint32_t ADC_RAW_TEMPERATURE;
    int32_t  DT;
    int64_t  T2, OFF2, SENS2, OFF, SENS;
    float    TEMPERATURE;
    float    PRESSURE;
    double   ALTITUDE;
    MS5611_Output_Sample_Rate PRESSURE_SAMPLE_RATE;
    MS5611_Output_Sample_Rate TEMPERATURE_SAMPLE_RATE;
} GebraBit_MS5611;
/*
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: */
/********************************************************
 *Declare Read&Write MS5611 Register Values Functions *
 ********************************************************/
uint8_t GB_MS5611_Burst_Read(uint8_t regAddr, uint8_t *data, uint16_t byteQuantity);
uint8_t GB_MS5611_Write_Reg_Data(uint8_t regAddr);
/********************************************************
 *       Declare MS5611 Configuration Functions       *
 ********************************************************/
void GB_MS5611_Soft_Reset(GebraBit_MS5611* MS5611);
void GB_MS5611_Read_PROM(GebraBit_MS5611* MS5611);
void GB_MS5611_Pressure_Sample_Rate(GebraBit_MS5611* MS5611, MS5611_Output_Sample_Rate rate);
void GB_MS5611_Temperature_Sample_Rate(GebraBit_MS5611* MS5611, MS5611_Output_Sample_Rate rate);
void GB_MS5611_Start_Pressure_Sampling(GebraBit_MS5611* MS5611);
void GB_MS5611_Start_Temperature_Sampling(GebraBit_MS5611* MS5611);
void GB_MS5611_initialize(GebraBit_MS5611* MS5611);
void GB_MS5611_Read_ADC(GebraBit_MS5611* MS5611);
void GB_MS5611_Read_ADC_Raw_Pressure(GebraBit_MS5611* MS5611);
void GB_MS5611_Read_ADC_Raw_Temperature(GebraBit_MS5611* MS5611);
void GB_MS5611_Calculate_Temperature(GebraBit_MS5611* MS5611);
void GB_MS5611_Calculate_Temperature_Compensated_Pressure(GebraBit_MS5611* MS5611);
void GB_MS5611_Altitude(GebraBit_MS5611* MS5611);

#endif  // _MS5611__H_
